==> pk-select-1.0.16.tgz: package/src/Select.js <==
import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import { Portal } from 'react-portal';
import isEmpty from 'is-empty';
import {
  DEFAULT_LABELKEY, SELECT_HEIGHT, SELECT_WIDTH,
  MAXIMUM_DROPDOWN_HEIGHT, MINIMUM_DROPDOWN_WIDTH,
  UP, DOWN, ESC, RETURN, TAB, DROPDOWN_ITEM_HEIGHT, ACTUAL_VALUE
} from './constants';
import Input from './Input';
import Menu from './Menu';
import {
  noop, getStateFromProps, isMinLengthMatched, constructMenuProps,
  getUpdatedActiveIndex, optionsMap, filterResultsFromOptions, getLabelKey, getInputValue
} from './utils';
import { isEqual, debounce } from './helpers';
import ArrowDown from './icons/arrow-down';

const getMenuHeight = (maxDropdownHeight, length) => Math.min(maxDropdownHeight, (length ? (length * DROPDOWN_ITEM_HEIGHT) + 15 : DROPDOWN_ITEM_HEIGHT));

const getOffset = (el) => {
  const rect = el.getBoundingClientRect().toJSON();
  const scrollLeft = window.pageXOffset || document.documentElement.scrollLeft;
  const scrollTop = window.pageYOffset || document.documentElement.scrollTop;
  return {
    ...rect,
    top: rect.top + scrollTop,
    left: rect.left + scrollLeft
  };
};

const getActiveIndex = (options = [], selected = false, props) => {
  const { labelKey, defaultFirstItemSelected, async } = props;
  if (!selected) {
    return defaultFirstItemSelected && options.length ? 0 : -1;
  }
  if (async) {
    return defaultFirstItemSelected ? 0 : -1;
  }
  let _selected = [...selected].shift();
  let selectedValue = getInputValue(_selected, labelKey);
  const index = options.findIndex(option => getInputValue(option, labelKey) === selectedValue);
  return index;
}

class Select extends PureComponent {
  state = getStateFromProps(this.props);

  componentDidMount() {
    this._listenResizeEvent();
  }

  UNSAFE_componentWillReceiveProps(nextProps) {
    const { labelKey, multiple, options } = nextProps;

    const selected = optionsMap([...nextProps.selected], { labelKey });

    const nextOptions = optionsMap([...options], { labelKey });

    const nextState = {}
    if (!isEqual(nextOptions, this.state.options)) {
      nextState.options = nextOptions;
      nextState.results = filterResultsFromOptions([...nextOptions], {
        ...nextProps,
        value: this.state.value,
      });
      nextState.activeIndex = getActiveIndex([...nextState.results], selected.length && selected[0], nextProps);
      nextState.activeItem = nextState.activeIndex === -1 ? null : nextState.results[nextState.activeIndex];
    }

    if (multiple !== this.props.multiple) {
      nextState.value = '';
    }

    // If new selections are passed via props, treat as a controlled input.
    if (selected && !isEqual(selected, this.state.selected)) {
      nextState.selected = [...selected];

      if (multiple) {
        this.setState(nextState);
        return;
      }
      nextState.value = selected.length ? getInputValue(selected[0], labelKey) : '';
    }

    // Truncate selections when in single-select mode.
    let newSelected = selected || this.state.selected;
    if (!multiple && newSelected.length > 1) {
      newSelected = newSelected.slice(0, 1);
      nextState.selected = [...newSelected];
      nextState.value = getInputValue(newSelected[0], labelKey);
    }

    this.setState(nextState);
  }

  componentDidUpdate(prevProps, prevState) {
    if (prevState.value !== this.state.value) {
      this.setState({
        results: filterResultsFromOptions([...this.state.options], {
          ...this.props,
          value: this.state.value
        })
      });
    }
  }

  componentWillUnmount() {
    this._revokeResizeListener();
  }

  _setInputRef = input => {
    this.input = input;
    this._setMenuPosition();
  }

  _setMenuRef = menu => {
    this.menu = menu;
  }

  _checkMenuPosition = () => {
    const menuPosition = this._getMenuPosition();
    if (!isEqual(menuPosition, this.state.menuPosition)) {
      this._setMenuPosition(menuPosition);
    }
  }

  _getMenuPosition = () => {
    if (!this.input) {
      return;
    }
    const menuHeight = getMenuHeight(this.props.maxDropdownHeight, this.state.results.length);
    const body = document.documentElement || document.body;
    const position = getOffset(this.input);
    let bottom = 'auto';
    let top = position.top + position.height + 2;
    let dropup = false;
    if (this.props.dropup || (this.props.flip && ((position.bottom + menuHeight) > (body.scrollHeight - body.scrollTop)))) {
      top = 'auto';
      bottom = (body.scrollHeight - body.scrollTop - position.top + 2);
      dropup = true;
    }
    return {
      top,
      bottom,
      left: position.left,
      width: position.width,
      dropup
    };
  }

  _setMenuPosition = (menuPosition = this._getMenuPosition()) => {
    this.setState(prevState => ({
      ...prevState,
      menuPosition
    }));
  }

  _handleInputChange = e => {
    e.persist();
    const value = e.currentTarget.value;
    const clearSelection = !this.props.multiple && this.state.selected.length;
    this.setState((prevState) => {
      return {
        isDirty: true,
        selected: clearSelection ? [] : prevState.selected,
        value,
      }
    }, () => {
      isMinLengthMatched(value, this.props) && this._openMenu();
      this.props.onInputChange && this.props.onInputChange(value, e);
      clearSelection && this.props.onChange([]);
    })
  }

  _handleActiveIndexChange = (activeIndex) => {
    this.setState((state) => ({
      activeIndex,
      activeItem: activeIndex === -1 ? null : state.results[activeIndex],
    }));
  }

  _handleKeyDown = (e) => {
    const { activeItem } = this.state;

    // Skip most actions when the menu is hidden.
    if (!this._isMenuOpen()) {
      if (e.keyCode === UP || e.keyCode === DOWN) {
        this._openMenu();
      }

      return;
    }

    switch (e.keyCode) {
      case UP:
      case DOWN:
        // Prevent input cursor from going to the beginning when pressing up.
        e.preventDefault();
        this._enableAutoScroll();
        this._handleActiveIndexChange(getUpdatedActiveIndex(
          this.state.activeIndex,
          e.keyCode,
          this.state.results
        ));
        break;
      case RETURN:
        // Prevent form submission while menu is open.
        e.preventDefault();
        activeItem && this._onMenuClick(activeItem);
        break;
      case ESC:
      case TAB:
        // ESC simply hides the menu. TAB will blur the input and move focus to
        // the next item hide the menu so it doesn't gain focus.
        this._closeMenu();
        break;
      default:
        break;
    }
  }

  _onFocus = e => {
    e.persist();
    if (!this._hasFocus()) {
      this._setFocus(true);
      (!this.props.searchable || isMinLengthMatched(this.state.value, this.props)) && this._openMenu();
    } else {
      console.log(this._isMenuOpen());
      (!this._isMenuOpen() && (!this.props.searchable || isMinLengthMatched(this.state.value, this.props))) && this._openMenu();
    }
    this.props.onInputFocus && this.props.onInputFocus(e);
  }

  _onBlur = e => {
    e.persist();
    this._hasFocus() && this._setFocus(false);
    this.props.onInputBlur && this.props.onInputBlur(e);
  }

  _onMenuClick = option => {
    const {
      [ACTUAL_VALUE]: selected
    } = option;
    this.props.onChange && this.props.onChange([selected]);
    this._closeMenu();
  }

  _hasFocus = () => this.state.focus;

  _isMenuOpen = () => this.state.open;

  _setFocus = focus => {
    this.setState({
      focus
    });
  }

  _triggerOpen = () => {
    if (!this._isMenuOpen()) {
      this.input && this.input.focus();
      this._openMenu();
    } else {
      this.input && this.input.blur();
      this._closeMenu();
    }
  }

  _openMenu = () => {
    this._checkMenuPosition();
    this._enableAutoScroll();
    this.setState(state => {
      const nextState = {
        open: true,
      };
      if (!state.searchable) {
        nextState.activeIndex = getActiveIndex(state.results, state.selected.length && state.selected, this.props);
      } else {
        nextState.activeIndex = this.props.defaultFirstItemSelected ? 0 : -1;
      }
      nextState.activeItem = nextState.activeIndex === -1 ? null : state.results[nextState.activeIndex];
      return nextState;
    }, () => {
      setTimeout(() => {
        this._listenClickEvent();
      }, 100);
    });
  }

  _closeMenu = () => {
    this._disableAutoScroll();
    this.setState({
      activeIndex: -1,
      activeItem: null,
      open: false
    }, () => {
      setTimeout(() => {
        this._revokeClickListener();
      }, 100);
    });
  }

  _listenResizeEvent = () => window.addEventListener('resize', this._handleResize);

  _revokeResizeListener = () => window.removeEventListener('resize', this._handleResize);

  _listenClickEvent = () => {
    document.addEventListener('click', this._handleOutsideClick, false);
    document.addEventListener('scroll', this._handlePageScroll, true);
  }

  _revokeClickListener = () => {
    document.removeEventListener('click', this._handleOutsideClick, false);
    document.addEventListener('scroll', this._handlePageScroll, true);
  }

  _handleOutsideClick = (event) => {
    if (this._isMenuOpen() && (this.menu && !this.menu.contains(event.target)) && (this.input !== event.target) && (this.icon && !this.icon.contains(event.target)) && this.props.closeOnOutsideClick) {
      this._closeMenu();
    }
  }

  _handleResize = () => {
    this._recomputeMenuPostition();
  };

  _handlePageScroll = (e) => {
    if (this.menu && !this.menu.contains(e.target)) {
      this._closeMenu();
    }
  };

  _recomputeMenuPostition = debounce(() => {
    this._setMenuPosition();
  }, 500);

  _autoScrollEnabled = () => this.state.autoScroll;

  _enableAutoScroll = () => {
    if (!this._autoScrollEnabled()) {
      this.setState({
        autoScroll: true
      });
    }
  };

  _disableAutoScroll = () => {
    if (this._autoScrollEnabled()) {
      this.setState({
        autoScroll: false
      });
    }
  };

  render() {
    const {
      id,
      width,
      height,
      inputProps,
      placeHolder,
      searchable,
      loading,
      icons,
    } = this.props;

    const {
      value,
      menuPosition,
      open
    } = this.state;

    const mergedStateAndProps = {
      ...this.props,
      ...this.state,
    };

    const rightIcon = {
      component: isEmpty(icons.right) ? <ArrowDown /> : icons.right,
      onClick: this._triggerOpen,
      additionalClasses: 'is-clickable',
      iconRef: icon => this.icon = icon
    };

    const leftIcon = {
      component: icons.left ? icons.left : null
    };

    return (
      <div className='pk-select' id={id} style={{ height, width }}>
        <Input
          {...inputProps}
          placeHolder={placeHolder ? placeHolder : (searchable ? 'Search...' : 'Select...')}
          value={value}
          onBlur={this._onBlur}
          onFocus={this._onFocus}
          onChange={searchable ? this._handleInputChange : noop}
          onKeyDown={this._handleKeyDown}
          readOnly={!searchable}
          inputRef={this._setInputRef}
          loading={loading}
          icons={{
            left: leftIcon,
            right: rightIcon
          }}
        />
        <Portal container={document.body}>
          <Menu
            {...constructMenuProps(mergedStateAndProps)}
            disableAutoScroll={this._disableAutoScroll}
            menuRef={this._setMenuRef}
            style={menuPosition}
            open={open}
            onMenuClick={this._onMenuClick}
          />
        </Portal>
      </div>
    )
  }
}

Select.propTypes = {
  options: PropTypes.array.isRequired,
  selected: PropTypes.array.isRequired,
  multiple: PropTypes.bool.isRequired,
  id: PropTypes.string.isRequired,
  onChange: PropTypes.func.isRequired,
  labelKey: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  searchable: PropTypes.bool,
  defaultFirstItemSelected: PropTypes.bool,
  onInputChange: PropTypes.func,
  onInputFocus: PropTypes.func,
  onInputBlur: PropTypes.func,
  minLength: PropTypes.number,
  height: PropTypes.number,
  maxDropdownHeight: PropTypes.number,
  inputProps: PropTypes.object,
  placeHolder: PropTypes.string,
  filterBy: PropTypes.oneOfType([PropTypes.array, PropTypes.func]),
  closeOnOutsideClick: PropTypes.bool,
  width: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  height: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  maxDropdownHeight: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  minDropdownWidth: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  icons: PropTypes.shape({
    left: PropTypes.any,
    right: PropTypes.any
  }),
  animate: PropTypes.bool,
};

Select.defaultProps = {
  options: [],
  selected: [],
  multiple: false,
  id: new Date().getTime().toString(),
  onChange: noop,
  searchable: false,
  minLength: 2,
  maxDropdownHeight: MAXIMUM_DROPDOWN_HEIGHT,
  minDropdownWidth: MINIMUM_DROPDOWN_WIDTH,
  inputProps: {},
  filterBy: [],
  labelKey: DEFAULT_LABELKEY,
  closeOnOutsideClick: true,
  width: SELECT_WIDTH,
  height: SELECT_HEIGHT,
  icons: {},
  animate: true,
}

export default Select

==> pk-select-1.0.17.tgz: package/src/Select.js <==
import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import Portal from 'react-overlays/Portal';
import isEmpty from 'is-empty';
import {
  DEFAULT_LABELKEY, SELECT_HEIGHT, SELECT_WIDTH,
  MAXIMUM_DROPDOWN_HEIGHT, MINIMUM_DROPDOWN_WIDTH,
  UP, DOWN, ESC, RETURN, TAB, DROPDOWN_ITEM_HEIGHT, ACTUAL_VALUE
} from './constants';
import Input from './Input';
import Menu from './Menu';
import {
  noop, getStateFromProps, isMinLengthMatched, constructMenuProps,
  getUpdatedActiveIndex, optionsMap, filterResultsFromOptions, getLabelKey, getInputValue
} from './utils';
import { isEqual, debounce } from './helpers';
import ArrowDown from './icons/arrow-down';

const getMenuHeight = (maxDropdownHeight, length) => Math.min(maxDropdownHeight, (length ? (length * DROPDOWN_ITEM_HEIGHT) + 15 : DROPDOWN_ITEM_HEIGHT));

const getOffset = (el) => {
  const rect = el.getBoundingClientRect().toJSON();
  const scrollLeft = window.pageXOffset || document.documentElement.scrollLeft;
  const scrollTop = window.pageYOffset || document.documentElement.scrollTop;
  return {
    ...rect,
    top: rect.top + scrollTop,
    left: rect.left + scrollLeft
  };
};

const getActiveIndex = (options = [], selected = false, props) => {
  const { labelKey, defaultFirstItemSelected, async } = props;
  if (!selected) {
    return defaultFirstItemSelected && options.length ? 0 : -1;
  }
  if (async) {
    return defaultFirstItemSelected ? 0 : -1;
  }
  let _selected = [...selected].shift();
  let selectedValue = getInputValue(_selected, labelKey);
  const index = options.findIndex(option => getInputValue(option, labelKey) === selectedValue);
  return index;
}

class Select extends PureComponent {
  state = getStateFromProps(this.props);

  componentDidMount() {
    this._listenResizeEvent();
  }

  UNSAFE_componentWillReceiveProps(nextProps) {
    const { labelKey, multiple, options } = nextProps;

    const selected = optionsMap([...nextProps.selected], { labelKey });

    const nextOptions = optionsMap([...options], { labelKey });

    const nextState = {}
    if (!isEqual(nextOptions, this.state.options)) {
      nextState.options = nextOptions;
      nextState.results = filterResultsFromOptions([...nextOptions], {
        ...nextProps,
        value: this.state.value,
      });
      nextState.activeIndex = getActiveIndex([...nextState.results], selected.length && selected[0], nextProps);
      nextState.activeItem = nextState.activeIndex === -1 ? null : nextState.results[nextState.activeIndex];
    }

    if (multiple !== this.props.multiple) {
      nextState.value = '';
    }

    // If new selections are passed via props, treat as a controlled input.
    if (selected && !isEqual(selected, this.state.selected)) {
      nextState.selected = [...selected];

      if (multiple) {
        this.setState(nextState);
        return;
      }
      nextState.value = selected.length ? getInputValue(selected[0], labelKey) : '';
    }

    // Truncate selections when in single-select mode.
    let newSelected = selected || this.state.selected;
    if (!multiple && newSelected.length > 1) {
      newSelected = newSelected.slice(0, 1);
      nextState.selected = [...newSelected];
      nextState.value = getInputValue(newSelected[0], labelKey);
    }

    this.setState(nextState);
  }

  componentDidUpdate(prevProps, prevState) {
    if (prevState.value !== this.state.value) {
      this.setState({
        results: filterResultsFromOptions([...this.state.options], {
          ...this.props,
          value: this.state.value
        })
      });
    }
  }

  componentWillUnmount() {
    this._revokeResizeListener();
  }

  _setInputRef = input => {
    this.input = input;
    this._setMenuPosition();
  }

  _setMenuRef = menu => {
    this.menu = menu;
  }

  _checkMenuPosition = () => {
    const menuPosition = this._getMenuPosition();
    if (!isEqual(menuPosition, this.state.menuPosition)) {
      this._setMenuPosition(menuPosition);
    }
  }

  _getMenuPosition = () => {
    if (!this.input) {
      return;
    }
    const menuHeight = getMenuHeight(this.props.maxDropdownHeight, this.state.results.length);
    const body = document.documentElement || document.body;
    const position = getOffset(this.input);
    let bottom = 'auto';
    let top = position.top + position.height + 2;
    let dropup = false;
    if (this.props.dropup || (this.props.flip && ((position.bottom + menuHeight) > (body.scrollHeight - body.scrollTop)))) {
      top = 'auto';
      bottom = (body.scrollHeight - body.scrollTop - position.top + 2);
      dropup = true;
    }
    return {
      top,
      bottom,
      left: position.left,
      width: position.width,
      dropup
    };
  }

  _setMenuPosition = (menuPosition = this._getMenuPosition()) => {
    this.setState(prevState => ({
      ...prevState,
      menuPosition
    }));
  }

  _handleInputChange = e => {
    e.persist();
    const value = e.currentTarget.value;
    const clearSelection = !this.props.multiple && this.state.selected.length;
    this.setState((prevState) => {
      return {
        isDirty: true,
        selected: clearSelection ? [] : prevState.selected,
        value,
      }
    }, () => {
      isMinLengthMatched(value, this.props) && this._openMenu();
      this.props.onInputChange && this.props.onInputChange(value, e);
      clearSelection && this.props.onChange([]);
    })
  }

  _handleActiveIndexChange = (activeIndex) => {
    this.setState((state) => ({
      activeIndex,
      activeItem: activeIndex === -1 ? null : state.results[activeIndex],
    }));
  }

  _handleKeyDown = (e) => {
    const { activeItem } = this.state;

    // Skip most actions when the menu is hidden.
    if (!this._isMenuOpen()) {
      if (e.keyCode === UP || e.keyCode === DOWN) {
        this._openMenu();
      }

      return;
    }

    switch (e.keyCode) {
      case UP:
      case DOWN:
        // Prevent input cursor from going to the beginning when pressing up.
        e.preventDefault();
        this._enableAutoScroll();
        this._handleActiveIndexChange(getUpdatedActiveIndex(
          this.state.activeIndex,
          e.keyCode,
          this.state.results
        ));
        break;
      case RETURN:
        // Prevent form submission while menu is open.
        e.preventDefault();
        activeItem && this._onMenuClick(activeItem);
        break;
      case ESC:
      case TAB:
        // ESC simply hides the menu. TAB will blur the input and move focus to
        // the next item hide the menu so it doesn't gain focus.
        this._closeMenu();
        break;
      default:
        break;
    }
  }

  _onClick = e => {
    e.persist();

    if (this._hasFocus() && !this._isMenuOpen()) {
      (!this.props.searchable || isMinLengthMatched(this.state.value, this.props)) && this._openMenu();
    }

    this.props.onInputClick && this.props.onInputClick();
  }

  _onFocus = e => {
    e.persist();
    if (!this._hasFocus()) {
      this._setFocus(true);
      (!this.props.searchable || isMinLengthMatched(this.state.value, this.props)) && this._openMenu();
    } else {
      console.log(this._isMenuOpen());
      (!this._isMenuOpen() && (!this.props.searchable || isMinLengthMatched(this.state.value, this.props))) && this._openMenu();
    }
    this.props.onInputFocus && this.props.onInputFocus(e);
  }

  _onBlur = e => {
    e.persist();
    this._hasFocus() && this._setFocus(false);
    this.props.onInputBlur && this.props.onInputBlur(e);
  }

  _onMenuClick = option => {
    const {
      [ACTUAL_VALUE]: selected
    } = option;
    this.props.onChange && this.props.onChange([selected]);
    this._closeMenu();
  }

  _hasFocus = () => this.state.focus;

  _isMenuOpen = () => this.state.open;

  _setFocus = focus => {
    this.setState({
      focus
    });
  }

  _triggerOpen = () => {
    if (!this._isMenuOpen()) {
      this.input && this.input.focus();
      this._openMenu();
    } else {
      this.input && this.input.blur();
      this._closeMenu();
    }
  }

  _openMenu = () => {
    this._checkMenuPosition();
    this._enableAutoScroll();
    this.setState(state => {
      const nextState = {
        open: true,
      };
      if (!state.searchable) {
        nextState.activeIndex = getActiveIndex(state.results, state.selected.length && state.selected, this.props);
      } else {
        nextState.activeIndex = this.props.defaultFirstItemSelected ? 0 : -1;
      }
      nextState.activeItem = nextState.activeIndex === -1 ? null : state.results[nextState.activeIndex];
      return nextState;
    }, () => {
      setTimeout(() => {
        this._listenClickEvent();
      }, 100);
    });
  }

  _closeMenu = () => {
    this._disableAutoScroll();
    this.setState({
      activeIndex: -1,
      activeItem: null,
      open: false
    }, () => {
      setTimeout(() => {
        this._revokeClickListener();
      }, 100);
    });
  }

  _listenResizeEvent = () => window.addEventListener('resize', this._handleResize);

  _revokeResizeListener = () => window.removeEventListener('resize', this._handleResize);

  _listenClickEvent = () => {
    document.addEventListener('click', this._handleOutsideClick, false);
    document.addEventListener('scroll', this._handlePageScroll, true);
  }

  _revokeClickListener = () => {
    document.removeEventListener('click', this._handleOutsideClick, false);
    document.addEventListener('scroll', this._handlePageScroll, true);
  }

  _handleOutsideClick = (event) => {
    if (this._isMenuOpen() && (this.menu && !this.menu.contains(event.target)) && this.props.closeOnOutsideClick) {
      this._closeMenu();
    }
  }

  _handleResize = () => {
    this._recomputeMenuPostition();
  };

  _handlePageScroll = (e) => {
    if (this.menu && !this.menu.contains(e.target)) {
      this._closeMenu();
    }
  };

  _recomputeMenuPostition = debounce(() => {
    this._setMenuPosition();
  }, 500);

  _autoScrollEnabled = () => this.state.autoScroll;

  _enableAutoScroll = () => {
    if (!this._autoScrollEnabled()) {
      this.setState({
        autoScroll: true
      });
    }
  };

  _disableAutoScroll = () => {
    if (this._autoScrollEnabled()) {
      this.setState({
        autoScroll: false
      });
    }
  };

  render() {
    const {
      id,
      width,
      height,
      inputProps,
      placeHolder,
      searchable,
      loading,
      icons,
    } = this.props;

    const {
      value,
      menuPosition,
      open
    } = this.state;

    const mergedStateAndProps = {
      ...this.props,
      ...this.state,
    };

    const rightIcon = {
      component: isEmpty(icons.right) ? <ArrowDown /> : icons.right,
      onClick: this._triggerOpen,
      additionalClasses: `is-clickable${open ? ' rotate-180deg' : ''}`,
    };

    const leftIcon = {
      component: icons.left ? icons.left : null
    };

    return (
      <div className='pk-select' id={id} style={{ height, width }}>
        <Input
          {...inputProps}
          placeHolder={placeHolder ? placeHolder : (searchable ? 'Search...' : 'Select...')}
          value={value}
          onClick={this._onClick}
          onBlur={this._onBlur}
          onFocus={this._onFocus}
          onChange={searchable ? this._handleInputChange : noop}
          onKeyDown={this._handleKeyDown}
          readOnly={!searchable}
          inputRef={this._setInputRef}
          loading={loading}
          icons={{
            left: leftIcon,
            right: rightIcon
          }}
        />
        <Portal container={document.body}>
          <Menu
            {...constructMenuProps(mergedStateAndProps)}
            disableAutoScroll={this._disableAutoScroll}
            menuRef={this._setMenuRef}
            style={menuPosition}
            open={open}
            onMenuClick={this._onMenuClick}
          />
        </Portal>
      </div>
    )
  }
}

Select.propTypes = {
  options: PropTypes.array.isRequired,
  selected: PropTypes.array.isRequired,
  multiple: PropTypes.bool.isRequired,
  id: PropTypes.string.isRequired,
  onChange: PropTypes.func.isRequired,
  labelKey: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  searchable: PropTypes.bool,
  defaultFirstItemSelected: PropTypes.bool,
  onInputChange: PropTypes.func,
  onInputFocus: PropTypes.func,
  onInputBlur: PropTypes.func,
  minLength: PropTypes.number,
  height: PropTypes.number,
  maxDropdownHeight: PropTypes.number,
  inputProps: PropTypes.object,
  placeHolder: PropTypes.string,
  filterBy: PropTypes.oneOfType([PropTypes.array, PropTypes.func]),
  closeOnOutsideClick: PropTypes.bool,
  width: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  height: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  maxDropdownHeight: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  minDropdownWidth: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  icons: PropTypes.shape({
    left: PropTypes.any,
    right: PropTypes.any
  }),
  animate: PropTypes.bool,
};

Select.defaultProps = {
  options: [],
  selected: [],
  multiple: false,
  id: new Date().getTime().toString(),
  onChange: noop,
  searchable: false,
  minLength: 2,
  maxDropdownHeight: MAXIMUM_DROPDOWN_HEIGHT,
  minDropdownWidth: MINIMUM_DROPDOWN_WIDTH,
  inputProps: {},
  filterBy: [],
  labelKey: DEFAULT_LABELKEY,
  closeOnOutsideClick: true,
  width: SELECT_WIDTH,
  height: SELECT_HEIGHT,
  icons: {},
  animate: true,
}

export default Select
